# Hey emacs! This is a -*- Perl -*- script!
# read_taginfo.pl -- Perl utility function to read Lintian's tag information

require "$LINTIAN_ROOT/lib/util.pl";
require "$LINTIAN_ROOT/lib/text_utils.pl";
require "$LINTIAN_ROOT/lib/manual_refs.pl";

# define hash for manuals
%manual = (
	   'policy' => 'Policy Manual',
	   'packaging' => 'Packaging Manual',
	   'devref' => 'Developers Reference',
	   'fsstnd' => 'FSSTND',
	   'fhs' => 'FHS',
	   );

srand;

# load information about checker scripts
sub read_tag_info {
  my ($type) = @_;

  my $dtml_convert;
  if ($type eq 'html') {
    $dtml_convert = \&dtml_to_html;
  } else {
    $dtml_convert = \&dtml_to_text;
  }

  for $f (<$LINTIAN_ROOT/checks/*.desc>) {
    print "N: Reading checker description file $f ...\n" if $debug >= 2;

    my @secs = read_dpkg_control($f);
    
    $secs[0]->{'check-script'} or fail("error in description file $f: `Check-Script:' not defined");
    
    for ($i=1; $i<=$#secs; $i++) {
      ($tag = $secs[$i]->{'tag'}) or fail("error in description file $f: section $i does not have a `Tag:'");
      
      $tag_data{$tag} = $secs[$i];

      my @foo = split_paragraphs($secs[$i]->{'info'});
      if ($secs[$i]->{'ref'}) {
	push(@foo,"");
	push(@foo,format_ref($secs[$i]->{'ref'}));
      }

      $tag_info{$tag} = join("\n",&$dtml_convert(@foo));
    }
  }
}

sub format_ref {
  my ($ref) = @_;

  my @foo = split(/\s*,\s*/o,$ref);
  my $u;
  for ($u=0; $u<=$#foo; $u++) {
    if ($foo[$u] =~ m,^\s*(policy|packaging|devref|fsstnd|fhs)\s*([\d\.]+)?\s*$,oi) {
      my ($man,$sec) = ($1,$2);
	    
      $foo[$u] = $manual{lc $man};
	    
      if ($sec =~ m,^\d+$,o) {
	$foo[$u] .= ", chapter $sec";
      } elsif ($sec) {
	$foo[$u] .= ", section $sec";
      }

      if (exists $url{"$man-$sec"}) {
	$foo[$u] = "<a href=\"$url{\"$man-$sec\"}\">$foo[$u]</a>";
      } elsif (exists $url{$man}) {
	$foo[$u] = "<a href=\"$url{$man}\">$foo[$u]</a>";
      }
    }
  }
	
  my $t = int(rand 4);
  if ($t == 0) {
    $text = "Refer to";
  } elsif ($t == 1) {
    $text = "Please check out";
  } elsif ($t == 2) {
    $text = "Have a look at";
  } elsif ($t == 3) {
    $text = "See";
  }
	
  if ($#foo+1 > 2) {
    $ref = sprintf "$text %s, and %s for details.",join(', ',splice(@foo,0,$#foo)),@foo;
  } elsif ($#foo+1 > 0) {
    $ref = sprintf "$text %s for details.",join(' and ',@foo);
  }
  
  return $ref;
}

1;
