# Hey emacs! This is a -*- Perl -*- script!
# text_utils.pl -- Perl utility functions for lintian

# requires wrap() function
use Text::Wrap;

# split_paragraphs -- splits a bunch of text lines into paragraphs.
# This function returns a list of paragraphs.
# Paragraphs are separated by empty lines. Each empty line is an own
# paragraph. Furthermore, indented lines are considered as own paragraph.
sub split_paragraphs {
  my $t = join("\n",@_);

  my ($l,@o);
  while ($t) {
    $t =~ s/^\.\n/\n/o;
    # starts with space or empty line?
    if (($t =~ s/^([ \t][^\n]*)\n?//o) or ($t =~ s/^()\n//o)) {
      #FLUSH;
      if ($l) {
	$l =~ s/\s+/ /go;
	$l =~ s/^\s+//o;
	$l =~ s/\s+$//o;
	push(@o,$l);
	undef $l;
      }
      #
      push(@o,$1);
    }
    # normal line?
    elsif ($t =~ s/^([^\n]*)\n?//o) {
      $l .= "$1 ";
    }
    # what else can happen?
    else {
      fail("internal error in wrapp");
    }
  }
  #FLUSH;
  if ($l) {
    $l =~ s/\s+/ /go;
    $l =~ s/^\s+//o;
    $l =~ s/\s+$//o;
    push(@o,$l);
    undef $l;
  }
  #
  
  return @o;
}

sub dtml_to_html {
  my @o;

  my $pre=0;
  for $_ (@_) {
    s,\&maint\;,<a href=\"mailto:schwarz\@monet.m.isar.de\">Christian Schwarz</a>,o;    # "
    s,\&debdev\;,<a href=\"mailto:debian-devel\@lists.debian.org\">debian-devel</a>,o;  # "
    
    # empty line?
    if (/^\s*$/o) {
      if ($pre) {
	push(@o,"\n");
      }
    }
    # preformatted line?
    elsif (/^\s/o) {
      if (not $pre) {
	push(@o,"<pre>");
	$pre=1;
      }
      push(@o,"$_");
    }
    # normal line
    else {
      if ($pre) {
	push(@o,"</pre>");
	$pre=0;
      }
      push(@o,"$_<p>\n");
    }
  }
  if ($pre) {
    push(@o,"</pre>");
    $pre=0;
  }

  return @o;
}

sub dtml_to_text {
  for $_ (@_) {
    # substitute Lintian &tags;
    s,&maint;,schwarz\@monet.m.isar.de,go;
    s,&debdev;,debian-devel\@lists.debian.org,go;

    # substitute HTML <tags>
    s,<i>,&lt;,go;
    s,</i>,&gt;,go;
    s,<[^>]+>,,go;

    # substitute HTML &tags;
    s,&lt;,<,go;
    s,&gt;,>,go;
    s,&amp;,\&,go;
    
    # preformatted?
    if (not /^\s/o) {
      # no.

      s,\s\s+, ,go;
      s,^ ,,o;
      s, $,,o;
    }
  }

  return @_;
}

# wrap_paragraphs -- wrap paragraphs in dpkg/dselect style.
# indented lines are not wrapped but displayed "as is"
sub wrap_paragraphs {
  my $a = shift;
  my $b = shift;

  my $o;
  for $t (split_paragraphs(@_)) {
    # indented line?
    if ($t =~ m,^\s,o) {
      $o .= "$a$t\n";
    } else {
      $o .= wrap($a,$b,"$t\n");
    }
  }
  
  return $o;
}

# ------------------------

sub fail {
  if ($_[0]) {
    print STDERR "error: $_[0]\n";
  } elsif ($!) {
    print STDERR "error: $!\n";
  } else {
    print STDERR "error.\n";
  }
  print STDERR "$_[1]\n" if $_[1];
  exit 1;
}

1;
